from ..cw_controller import CWController
# Class for /schedule/entries
from connectpyse.schedule import schedule_entry


class ScheduleEntriesAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'schedule'
        self.module = 'entries'
        self._class = schedule_entry.ScheduleEntry
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_schedule_entries(self):
        return super()._get()

    def create_schedule_entry(self, a_entry):
        return super()._create(a_entry)

    def get_schedule_entries_count(self):
        return super()._get_count()

    def get_schedule_entry_by_id(self, entry_id):
        return super()._get_by_id(entry_id)

    def delete_schedule_entry_by_id(self, entry_id):
        super()._delete_by_id(entry_id)

    def replace_schedule_entry(self, entry_id):
        pass

    def update_schedule_entry(self, entry_id, key, value):
        return super()._update(entry_id, key, value)

    def merge_schedule_entry(self, a_entry, target_entry_id):
        # return super()._merge(a_schedule_entry, target_schedule_entry_id)
        pass



from ..cw_controller import CWController
# Class for /company/contacts
from . import contact


class ContactsAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'company'
        self.module = 'contacts'
        self._class = contact.Contact
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_contacts(self):
        return super()._get()

    def create_contact(self, a_contact):
        return super()._create(a_contact)

    def get_contacts_count(self):
        return super()._get_count()

    def get_contact_by_id(self, contact_id):
        return super()._get_by_id(contact_id)

    def delete_contact_by_id(self, contact_id):
        super()._delete_by_id(contact_id)

    def replace_contact(self, contact_id):
        pass

    def update_contact(self, contact_id, key, value):
        return super()._update(contact_id, key, value)

    def merge_contact(self, a_contact, target_contact_id):
        # return super()._merge(a_contact, target_contact_id)
        pass

    def validate_portal_credentials(self, a_contact):
        for comm_items in a_contact.communicationItems:
            if comm_items['defaultFlag'] and comm_items['communicationType'] == 'Email':
                email = comm_items['value']
                break
        dict_post = {'email': email, 'password': a_contact.portalPassword}
        json_results = self.contacts.post(user_data=dict_post, user_headers=connectpyse.basic_auth)
        status_code = json_results.status_code
        return status_code




from ..cw_model import CWModel


class Info(CWModel):

    def __init__(self, json_dict=None):
        self.version = None  # (String)
        self.isCloud = None  # (Boolean)
        self.serverTimeZone = None  # (String)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ProjectTeamMember(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.projectId = None  # (Integer)
        self.hours = None  # (Number)
        self.member = None  # *(MemberReference)
        self.projectRole = None  # *(ProjectRoleReference)
        self.workRole = None  # (WorkRoleReference)
        self.startDate = None  # (String)
        self.endDate = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Agreement(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(100))
        self.type = None  # *(AgreementTypeReference)
        self.company = None  # *(CompanyReference)
        self.contact = None  # *(ContactReference)
        self.subContractCompany = None  # (CompanyReference)
        self.subContractContact = None  # (ContactReference)
        self.parentAgreementId = None  # (Integer)
        self.customerPO = None  # (String(50))
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.restrictLocationFlag = None  # (Boolean)
        self.restrictDepartmentFlag = None  # (Boolean)
        self.startDate = None  # (String)
        self.endDate = None  # (String)
        self.noEndingDateFlag = None  # (Boolean)
        self.opportunity = None  # (OpportunityReference)
        self.cancelledFlag = None  # (Boolean)
        self.dateCancelled = None  # (String)
        self.reasonCancelled = None  # (String(50))
        self.slaId = None  # (Integer)
        self.workOrder = None  # (String(20))
        self.internalNotes = None  # (String(5000))
        self.applicationUnits = None  # (Enum)
        self.applicationLimit = None  # (Number)
        self.applicationCycle = None  # (Enum)
        self.applicationUnlimitedFlag = None  # (Boolean)
        self.oneTimeFlag = None  # (Boolean)
        self.coverAgreementTime = None  # (Boolean)
        self.coverAgreementProduct = None  # (Boolean)
        self.coverAgreementExpense = None  # (Boolean)
        self.coverSalesTax = None  # (Boolean)
        self.carryOverUnused = None  # (Boolean)
        self.allowOverruns = None  # (Boolean)
        self.expiredDays = None  # (Integer)
        self.limit = None  # (Integer)
        self.expireWhenZero = None  # (Boolean)
        self.chargeToFirm = None  # (Boolean)
        self.employeeCompRate = None  # (Enum)
        self.employeeCompNotExceed = None  # (Enum)
        self.compHourlyRate = None  # (Number)
        self.compLimitAmount = None  # (Number)
        self.billCycleId = None  # (Integer)
        self.billOneTimeFlag = None  # (Boolean)
        self.billTermsId = None  # (Integer)
        self.invoicingCycle = None  # (Enum)
        self.billToCompany = None  # (CompanyReference)
        self.billToSite = None  # (SiteReference)
        self.billAmount = None  # (Number)
        self.taxable = None  # (Boolean)
        self.prorateFirstBill = None  # (Number)
        self.billStartDate = None  # (String)
        self.taxCodeId = None  # (Integer)
        self.restrictDownPayment = None  # (Boolean)
        self.invoiceDescription = None  # (String)
        self.topComment = None  # (Boolean)
        self.bottomComment = None  # (Boolean)
        self.workRole = None  # (WorkRoleReference)
        self.workType = None  # (WorkTypeReference)
        self.projectTypeId = None  # (Integer)
        self.invoiceTemplateSetupId = None  # (Integer)
        self.billTime = None  # (Enum)
        self.billExpenses = None  # (Enum)
        self.billProducts = None  # (Enum)
        self.billableTimeInvoice = None  # (Boolean)
        self.billableExpenseInvoice = None  # (Boolean)
        self.billableProductInvoice = None  # (Boolean)
        self._info = None  # (Metadata)
        self.periodType = None  # (Enum)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /company/companies/types
from connectpyse.company import company_type


class CompanyTypeAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'company'
        self.module = 'companies/types'
        self._class = company_type.CompanyType
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_company_types(self):
        return super()._get()

    def create_company_type(self, a_company_type):
        return super()._create(a_company_type)

    def get_company_types_count(self):
        return super()._get_count()

    def get_company_type_by_id(self, company_type_id):
        return super()._get_by_id(company_type_id)

    def delete_company_type_by_id(self, company_type_id):
        super()._delete_by_id(company_type_id)

    def replace_company_type(self, company_type_id):
        pass

    def update_company_type(self, company_type_id, key, value):
        return super()._update(company_type_id, key, value)

    def merge_company_type(self, a_company_type, target_company_type_id):
        # return super()._merge(a_company_type, target_company_type_id)
        pass



from ..cw_model import CWModel


class Contact(CWModel):
    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.firstName = None  # *(String(30))
        self.lastName = None  # (String(30))
        self.type = None  # (ContactTypeReference)
        self.company = None  # (CompanyReference)
        self.site = None  # (SiteReference)
        self.addressLine1 = None  # (String(50))
        self.addressLine2 = None  # (String(50))
        self.city = None  # (String(50))
        self.state = None  # (String(50))
        self.zip = None  # (String(12))
        self.country = None  # (String(50))
        self.relationship = None  # (RelationshipReference)
        self.department = None  # (ContactDepartmentReference)
        self.inactiveFlag = None  # (Boolean)
        self.defaultMergeContactId = None  # (Integer)
        self.securityIdentifier = None  # (String(184))
        self.managerContactId = None  # (Integer)
        self.assistantContactId = None  # (Integer)
        self.title = None  # (String(100))
        self.school = None  # (String(50))
        self.nickName = None  # (String(30))
        self.marriedFlag = None  # (Boolean)
        self.childrenFlag = None  # (Boolean)
        self.significantOther = None  # (String(30))
        self.portalPassword = None  # (String(15))
        self.portalSecurityLevel = None  # (Integer)
        self.disablePortalLoginFlag = None  # (Boolean)
        self.unsubscribeFlag = None  # (Boolean)
        self.gender = None  # (Enum)
        self.birthDay = None  # (String)
        self.anniversary = None  # (String)
        self.presence = None  # (Enum)
        self.mobileGuid = None  # (Guid)
        self.facebookUrl = None  # (String)
        self.twitterUrl = None  # (String)
        self.linkedInUrl = None  # (String)
        self.defaultBillingFlag = None  # (Boolean)
        self.communicationItems = None  # (Array)
        self._info = None  # (Metadata)
        self.customFields = None  # (Array)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class TicketTask(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.ticketId = None  # (Integer)
        self.notes = None  # (String)
        self.closedFlag = None  # (Boolean)
        self.priority = None  # (Integer)
        self.schedule = None  # (ScheduleEntryReference)
        self.code = None  # (ServiceCodeReference)
        self.resolution = None  # (String)
        self.childScheduleAction = None  # (Enum)
        self.childTicketId = None  # (Integer)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AgreementWorkType(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.workType = None  # (WorkTypeReference)
        self.locationId = None  # (Integer)
        self.rateType = None  # *(Enum)
        self.billTime = None  # *(Enum)
        self.rate = None  # (Number)
        self.hoursMax = None  # (Number)
        self.hoursMin = None  # (Number)
        self.roundBillHours = None  # (Number)
        self.overageRate = None  # (Number)
        self.overageRateType = None  # (Enum)
        self.agreementLimit = None  # (Number)
        self.site = None  # (SiteReference)
        self.effectiveDate = None  # (String)
        self.endingDate = None  # (String)
        self.agreementId = None  # (Integer)
        self.company = None  # (CompanyReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /company/companies
from . import company


class CompaniesAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'company'
        self.module = 'companies'
        self._class = company.Company
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_companies(self):
        return super()._get()

    def create_company(self, a_company):
        return super()._create(a_company)

    def get_companies_count(self):
        return super()._get_count()

    def get_company_by_id(self, company_id):
        return super()._get_by_id(company_id)

    def delete_company_by_id(self, company_id):
        super()._delete_by_id(company_id)

    def replace_company(self, company_id):
        pass

    def update_company(self, company_id, key, value):
        return super()._update(company_id, key, value)

    def merge_company(self, a_company, target_company_id):
        # return super()._merge(a_company, target_company_id)
        pass



from ..cw_controller import CWController
# Class for /time/entries
from . import time_entry


class TimeEntriesAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'time'
        self.module = 'entries'
        self._class = time_entry.TimeEntry
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_time_entries(self):
        return super()._get()

    def create_time_entry(self, a_time_entry):
        return super()._create(a_time_entry)

    def get_time_entries_count(self):
        return super()._get_count()

    def get_time_entry_by_id(self, time_entry_id):
        return super()._get_by_id(time_entry_id)

    def delete_time_entry_by_id(self, time_entry_id):
        super()._delete_by_id(time_entry_id)

    def replace_time_entry(self, time_entry_id):
        pass

    def update_time_entry(self, time_entry_id, key, value):
        return super()._update(time_entry_id, key, value)


from ..cw_model import CWModel


class ContactCommunication(CWModel):
    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.contactId = None  # (Integer)
        self.type = None  # *(CommunicationTypeReference)  {id (integer)(int32)), name (string)), _info}
        self.value = None  # *(String(250))
        self.extension = None  # (String(15))
        self.defaultFlag = None  # (Boolean)
        self.mobileGuid = None  # (Guid)
        self.communicationType = None  # (Enum)  ("Phone", "Fax", "Email")
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ProductItem(CWModel):

    def __init__(self, json_dict=None):
        # With: id(Integer)
        # Search: ^(\s*)(\*?)([_a-z0-9]*)\s?(\(.*$)
        # Replace: $1self.$3 = None  # $2$4
        # To get: self.id = None  # (Integer)
        self.id = None  # (Integer)
        self.catalogItem = None  # *(CatalogItemReference)
        self.chargeToId = None  # *(Integer)
        self.chargeToType = None  # *(Enum)
        self.description = None  # (String(2000))
        self.sequenceNumber = None  # (Number)
        self.quantity = None  # (Number)
        self.price = None  # (Number)
        self.cost = None  # (Number)
        self.discount = None  # (Number)
        self.priceMethod = None  # (Enum)
        self.billableOption = None  # *(Enum)
        self.agreement = None  # (AgreementReference)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.vendor = None  # (CompanyReference)
        self.vendorSku = None  # (String(50))
        self.taxableFlag = None  # (Boolean)
        self.dropshipFlag = None  # (Boolean)
        self.specialOrderFlag = None  # (Boolean)
        self.phaseProductFlag = None  # (Boolean)
        self.cancelledFlag = None  # (Boolean)
        self.quantityCancelled = None  # (Number)
        self.cancelledReason = None  # (String(100))
        self.customerDescription = None  # (String(6000))
        self.internalNotes = None  # (String(1000))
        self.productSuppliedFlag = None  # (Boolean)
        self.subContractorShipToId = None  # (Integer)
        self.subContractorAmountLimit = None  # (Number)
        self.recurring = None  # (ProductRecurring)
        self.sla = None  # (SLAReference)
        self.entityType = None  # (EntityTypeReference)
        self.forecastDetailId = None  # (Integer)
        self.cancelledBy = None  # (Integer)
        self.cancelledDate = None  # (String)
        self.warehouse = None  # (String)
        self.warehouseBin = None  # (String)
        self.purchaseDate = None  # (String)
        self.integrationXRef = None  # (String)
        self.listPrice = None  # (Number)
        self.serialNumberIds = None  # (Integer[])
        self.company = None  # (CompanyReference)
        self.forecastStatus = None  # (OpportunityStatusReference)
        self._info = None  # (Metadata)
        self.bypassForecastUpdate = None  # (Boolean)
        self.customFields = None  # (CustomFieldValue[])

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class CompanyManagementSummaryReport(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.managementSolution = None  # (ManagementSolutionReference)
        self.groupIdentifier = None  # *(String(100))
        self.deviceType = None  # (Enum)
        self.agreement = None  # (AgreementReference)
        self.snmpMachines = None  # (Integer)
        self.totalWorkstations = None  # (Integer)
        self.totalServers = None  # (Integer)
        self.totalWindowsServers = None  # (Integer)
        self.totalWindowsWorkstations = None  # (Integer)
        self.totalManagedMachines = None  # (Integer)
        self.serversOffline = None  # (Integer)
        self.serversDiskSpaceLow = None  # (Integer)
        self.failedBackupJobs = None  # (Integer)
        self.totalNotifications = None  # (Integer)
        self.successfulBackupJobs = None  # (Integer)
        self.serverAvailability = None  # (Integer)
        self.virusesRemoved = None  # (Integer)
        self.spywareItemsRemoved = None  # (Integer)
        self.windowsPatchesInstalled = None  # (Integer)
        self.diskCleanups = None  # (Integer)
        self.diskDefragmentations = None  # (Integer)
        self.fullyPatchedMachines = None  # (Integer)
        self.missingOneTwoPatchesMachines = None  # (Integer)
        self.missingThreeFivePatchesMachines = None  # (Integer)
        self.missingMoreFivePatchesMachines = None  # (Integer)
        self.missingUnscannedPatchesMachines = None  # (Integer)
        self.alertsGenerated = None  # (String)
        self.internetConnectivity = None  # (Number)
        self.diskSpaceCleanedMb = None  # (Integer)
        self.missingSecurityPatches = None  # (String)
        self.cpuUtilization = None  # (Number)
        self.memoryUtilization = None  # (Number)
        self.company = None  # (CompanyReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class InvoicePayment(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.type = None  # (String)
        self.invoice = None  # (InvoiceReference)
        self.amount = None  # *(Number)
        self.paymentDate = None  # (String)
        self.appliedBy = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ChargeCode(CWModel):

    def __init__(self, json_dict=None):
        # With: id(Integer)
        # Search: ^(\s*)(\*?)([_a-z0-9]*)\s?(\(.*$)
        # Replace: $1self.$3 = None  # $2$4
        # To get: self.id = None  # (Integer)
        self.id = None  # (Integer)
        self.name = None  # *(String(50))
        self.company = None  # *(CompanyReference)
        self.location = None  # (SystemLocationReference)
        self.department = None  # (SystemDepartmentReference)
        self.billTime = None  # (Enum)
        self.expenseEntryFlag = None  # (Boolean)
        self.allowAllExpenseTypeFlag = None  # (Boolean)
        self.timeEntryFlag = None  # (Boolean)
        self.workType = None  # *(WorkTypeReference)
        self.workRole = None  # (WorkRoleReference)
        self.integrationXref = None  # (String(50))
        self.expenseTypeIds = None  # (Integer[])
        self._info = None  # (Metadata)
        
        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class AgreementType(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(50))
        self.prefixSuffixOption = None  # *(Enum)
        self.defaultFlag = None  # (Boolean)
        self.inactiveFlag = None  # (Boolean)
        self.prePaymentFlag = None  # (Boolean)
        self.invoicePreSuffix = None  # (String(5))
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.restrictLocationFlag = None  # (Boolean)
        self.restrictDepartmentFlag = None  # (Boolean)
        self.slaId = None  # (Integer)
        self.applicationUnits = None  # (Enum)
        self.applicationLimit = None  # (Number)
        self.applicationCycle = None  # (Enum)
        self.applicationUnlimitedFlag = None  # (Boolean)
        self.oneTimeFlag = None  # (Boolean)
        self.coverAgreementTimeFlag = None  # (Boolean)
        self.coverAgreementProductFlag = None  # (Boolean)
        self.coverAgreementExpenseFlag = None  # (Boolean)
        self.coverSalesTaxFlag = None  # (Boolean)
        self.carryOverUnusedFlag = None  # (Boolean)
        self.allowOverrunsFlag = None  # (Boolean)
        self.expiredDays = None  # (Integer)
        self.limit = None  # (Integer)
        self.expireWhenZero = None  # (Boolean)
        self.chargeToFirmFlag = None  # (Boolean)
        self.employeeCompRate = None  # *(Enum)
        self.employeeCompNotExceed = None  # *(Enum)
        self.compHourlyRate = None  # (Number)
        self.compLimitAmount = None  # (Number)
        self.billCycleId = None  # (Integer)
        self.billOneTimeFlag = None  # (Boolean)
        self.billTermsId = None  # (Integer)
        self.invoicingCycle = None  # *(Enum)
        self.billAmount = None  # (Number)
        self.taxableFlag = None  # (Boolean)
        self.restrictDownPaymentFlag = None  # (Boolean)
        self.invoiceDescription = None  # (String(4000))
        self.topCommentFlag = None  # (Boolean)
        self.bottomCommentFlag = None  # (Boolean)
        self.workRole = None  # (WorkRoleReference)
        self.workType = None  # (WorkTypeReference)
        self.projectTypeId = None  # (Integer)
        self.invoiceTemplateSetupId = None  # (Integer)
        self.billTime = None  # *(Enum)
        self.billExpenses = None  # *(Enum)
        self.billProducts = None  # *(Enum)
        self.billableTimeInvoiceFlag = None  # (Boolean)
        self.billableExpenseInvoiceFlag = None  # (Boolean)
        self.billableProductInvoiceFlag = None  # (Boolean)
        self.copyWorkRolesFlag = None  # (Boolean)
        self.copyWorkTypesFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class TicketNote(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.ticketId = None  # (Integer)
        self.text = None  # (String)
        self.detailDescriptionFlag = None  # (Boolean)
        self.internalAnalysisFlag = None  # (Boolean)
        self.resolutionFlag = None  # (Boolean)
        self.member = None  # (MemberReference)
        self.contact = None  # (ContactReference)
        self.customerUpdatedFlag = None  # (Boolean)
        self.processNotifications = None  # (Boolean)
        self.dateCreated = None  # (String)
        self.createdBy = None  # (String)
        self.internalFlag = None  # (Boolean)
        self.externalFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Company(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.identifier = None  # *(String(25))
        self.name = None  # *(String(50))
        self.status = None  # *(CompanyStatusReference)
        self.type = None  # *(CompanyTypeReference)
        self.addressLine1 = None  # *(String(50))
        self.addressLine2 = None  # *(String(50))
        self.city = None  # *(String(50))
        self.state = None  # *(String(50))
        self.zip = None  # *(String(12))
        self.country = None  # (CountryReference)
        self.phoneNumber = None  # (String(30))
        self.faxNumber = None  # (String(30))
        self.website = None  # (String(255))
        self.territoryId = None  # (Integer)
        self.marketId = None  # (Integer)
        self.accountNumber = None  # (String(100))
        self.defaultContact = None  # (ContactReference)
        self.dateAcquired = None  # (String)
        self.sicCode = None  # (SicCodeReference)
        self.parentCompany = None  # (CompanyReference)
        self.annualRevenue = None  # (Number)
        self.numberOfEmployees = None  # (Integer)
        self.ownershipType = None  # (OwnershipTypeReference)
        self.timeZone = None  # (TimeZoneReference)
        self.leadSource = None  # (String(50))
        self.leadFlag = None  # (Boolean)
        self.unsubscribeFlag = None  # (Boolean)
        self.calendarId = None  # (Integer)
        self.userDefinedField1 = None  # (String(50))
        self.userDefinedField2 = None  # (String(50))
        self.userDefinedField3 = None  # (String(50))
        self.userDefinedField4 = None  # (String(50))
        self.userDefinedField5 = None  # (String(50))
        self.userDefinedField6 = None  # (String(50))
        self.userDefinedField7 = None  # (String(50))
        self.userDefinedField8 = None  # (String(50))
        self.userDefinedField9 = None  # (String(50))
        self.userDefinedField10 = None  # (String(50))
        self.vendorIdentifier = None  # (String(100))
        self.taxIdentifier = None  # (String)
        self.taxCode = None  # (TaxCodeReference)
        self.billingTerms = None  # (BillingTermsReference)
        self.invoiceTemplate = None  # (InvoiceTemplateReference)
        self.pricingSchedule = None  # (PricingScheduleReference)
        self.companyEntityType = None  # (EntityTypeReference)
        self.billToCompany = None  # (CompanyReference)
        self.billingSite = None  # (SiteReference)
        self.billingContact = None  # (ContactReference)
        self.invoiceDeliveryMethod = None  # (BillingDeliveryReference)
        self.invoiceToEmailAddress = None  # (String)
        self.invoiceCCEmailAddress = None  # (String)
        self.deletedFlag = None  # (Boolean)
        self.dateDeleted = None  # (String)
        self.deletedBy = None  # (String)
        self.mobileGuid = None  # (Guid)
        self.currency = None  # (CurrencyReference)
        self.territoryManager = None  # (MemberReference)
        self._info = None  # (Metadata)
        self.customFields = None  # (Array)

        # initialize object with json dict
        super().__init__(json_dict)





from ..cw_controller import CWController
# Class for /sales/orders
from . import order


class OrderAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'sales'
        self.module = 'orders'
        self._class = order.Order
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_orders(self):
        return super()._get()

    def create_order(self, a_order):
        return super()._create(a_order)

    def get_orders_count(self):
        return super()._get_count()

    def get_order_by_id(self, order_id):
        return super()._get_by_id(order_id)

    def delete_order_by_id(self, order_id):
        super()._delete_by_id(order_id)

    def replace_order(self, order_id):
        pass

    def update_order(self, order_id, key, value):
        return super()._update(order_id, key, value)


# -*- encoding: utf-8 -*-
#
# THE RESTAPI GENERIC CLIENT CONSUMER
# Author: Sergio Berlotto <sergio.berlotto@gmail.com>
#

# Project died, so updates made by me: Joshua M Smith <saether@gmail.com>


import requests as req
import json


class EndpointError(AttributeError):

    def __init__(self, endpoint):

        message = "Call Error: endpoint '{}' not allowed.".format(
            endpoint
        )
        super(EndpointError, self).__init__(message)


class ApiError(Exception):

    def __init__(self, method, route, status_code, server_message):

        message = """{method} {route} Return code: {status_code}\n
Server message:{server_message}""".format(**locals())
        super(ApiError, self).__init__(message)


class Client(object):

    def __init__(self, root_path):
        self.root_path = root_path
        self.known_endpoints = []

    def __getattribute__(self, attr):

        try:
            return super(Client, self).__getattribute__(attr)
        except AttributeError:
            return Endpoint(
                self.root_path,
                attr
            )


class Endpoint(object):

    def __init__(self, root_path, endpoint):
        self.endpoint = endpoint
        self.root_path = root_path

    def _url(self, path, *args):
        url = "{}/{}/".format(self.root_path, path)
        if args:
            url += "/".join([str(a) for a in args])
        return url

    def _headers(self, others={}):
        """Return the default headers and others as necessary"""
        headers = {
            'Content-Type': 'application/json'
        }

        for p in others.keys():
            headers[p] = others[p]
        return headers

    def _params(self, others={}):
        params = {}
        for p in others.keys():
            params[p] = others[p]
        return params

    def _formatreturn(self, resp):
        try:
            r = resp.json()
        except ValueError:
            #When the return is not a JSON
            r = resp.text
        return r

    def post(self, user_data, the_id=None, user_params={}, user_headers={}):

        strjsondata = json.dumps(user_data, ensure_ascii=False)

        if the_id:
            url = self._url(self.endpoint, the_id)
        else:
            url = self._url(self.endpoint)

        resp = req.post(
            url,
            data=strjsondata,
            headers=self._headers(user_headers),
            params=self._params(user_params),
            stream=False
        )

        if resp.status_code != 201:
            raise ApiError(
                "GET",
                self.endpoint,
                resp.status_code,
                resp.text)
        else:
            return self._formatreturn(resp)

    def put(self, the_id, user_data, user_params={}, user_headers={}):

        strjsondata = json.dumps(user_data, ensure_ascii=False)

        resp = req.put(
            self._url(self.endpoint, the_id),
            data=strjsondata,
            headers=self._headers(user_headers),
            params=self._params(user_params)
        )

        if resp.status_code != 200:
            raise ApiError(
                "GET",
                self.endpoint,
                resp.status_code,
                resp.text)
        else:
            return self._formatreturn(resp)

    def patch(self, the_id, user_data, user_params={}, user_headers={}):

        strjsondata = json.dumps(user_data, ensure_ascii=False)

        resp = req.patch(
            self._url(self.endpoint, the_id),
            data=strjsondata,
            headers=self._headers(user_headers),
            params=self._params(user_params)
        )

        if resp.status_code != 200:
            raise ApiError(
                "GET",
                self.endpoint,
                resp.status_code,
                resp.text)
        else:
            return self._formatreturn(resp)

    def get(self, the_id=None, level=None, user_params={}, user_headers={}):

        if the_id:
            url = self._url(self.endpoint, the_id)
        else:
            url = self._url(self.endpoint)

        resp = req.get(
            url,
            headers=self._headers(user_headers),
            params=self._params(user_params)
        )

        if resp.status_code != 200:
            raise ApiError(
                "GET",
                self.endpoint,
                resp.status_code,
                resp.text)
        else:
            return self._formatreturn(resp)

    def delete(self, level=None, user_params={}, user_headers={}):

        resp = req.delete(
            self._url(self.endpoint),
            headers=self._headers(user_headers),
            params=self._params(user_params)
        )

        if resp.status_code != 200:
            raise ApiError(
                "DELETE",
                self.endpoint,
                resp.status_code,
                resp.text)
        else:
            return self._formatreturn(resp)



from ..cw_model import CWModel


class Configuration(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # (String(100))*
        self.type = None  # (ConfigurationTypeReference)*
        self.status = None  # (ConfigurationStatusReference)
        self.company = None  # (CompanyReference)*
        self.contact = None  # (ContactReference)
        self.site = None  # (SiteReference)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.deviceIdentifier = None  # (String(100))
        self.serialNumber = None  # (String(250))
        self.modelNumber = None  # (String(50))
        self.tagNumber = None  # (String(50))
        self.purchaseDate = None  # (String)
        self.installationDate = None  # (String)
        self.installedBy = None  # (MemberReference)
        self.warrantyExpirationDate = None  # (String)
        self.vendorNotes = None  # (String)
        self.notes = None  # (String)
        self.macAddress = None  # (String(25))
        self.lastLoginName = None  # (String(100))
        self.billFlag = None  # (Boolean)
        self.backupSuccesses = None  # (Integer)
        self.backupIncomplete = None  # (Integer)
        self.backupFailed = None  # (Integer)
        self.backupRestores = None  # (Integer)
        self.lastBackupDate = None  # (String)
        self.backupServerName = None  # (String(50))
        self.backupBillableSpaceGb = None  # (Number)
        self.backupProtectedDeviceList = None  # (String)
        self.backupYear = None  # (Integer)
        self.backupMonth = None  # (Integer)
        self.ipAddress = None  # (String(50))
        self.defaultGateway = None  # (String(50))
        self.osType = None  # (String(250))
        self.osInfo = None  # (String(250))
        self.cpuSpeed = None  # (String(100))
        self.ram = None  # (String(25))
        self.localHardDrives = None  # (String)
        self.parentConfigurationId = None  # (Integer)
        self.vendor = None  # (CompanyReference)
        self.manufacturer = None  # (ManufacturerReference)
        self.questions = None  # (Array)
        self.activeFlag = None  # (Boolean)
        self.managementLink = None  # (String(200))
        self.remoteLink = None  # (String(200))
        self.sla = None  # (SLAReference)
        self.mobileGuid = None  # (Guid)
        self._info  = None  # (Metadata)
        self.customFields = None  # (Array)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ContactTrack(CWModel):
    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.trackId = None  # *(Integer)
        self.name = None  # (String)
        self.startDate = None  # (String)
        self.endDate = None  # (String)
        self.actionTaken = None  # (Integer)
        self.actionRemaining = None  # (Integer)
        self.startedBy = None  # (String)
        self.company = None  # (CompanyReference)
        self.contact = None  # (ContactReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Campaign(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(50))
        self.type = None  # *(CampaignTypeReference)
        self.subType = None  # *(CampaignSubTypeReference)
        self.status = None  # (CampaignStatusReference)
        self.startDate = None  # *(String)
        self.endDate = None  # (String)
        self.locationId = None  # (Integer)
        self.member = None  # (MemberReference)
        self.inactive = None  # (Boolean)
        self.inactiveDaysAfterEnd = None  # (Integer)
        self.notes = None  # (String)
        self.defaultGroup = None  # (GroupReference)
        self.marketingManagerDefaultTrackId = None  # (Integer)
        self.opportunityDefaultTrackId = None  # (Integer)
        self.impressions = None  # (Integer)
        self.budgetRevenue = None  # (Number)
        self.budgetCost = None  # (Number)
        self.actualCost = None  # (Number)
        self.budgetGrossMargin = None  # (Number)
        self.budgetROI = None  # (Number)
        self.actualRevenue = None  # (Number)
        self.actualGrossMargin = None  # (Number)
        self.actualROI = None  # (Number)
        self.emailsSent = None  # (Integer)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class CompanySite(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(50))
        self.addressLine1 = None  # (String(50))
        self.addressLine2 = None  # (String(50))
        self.city = None  # (String(50))
        self.state = None  # (String(50))
        self.zip = None  # (String(12))
        self.country = None  # (CountryReference)
        self.phoneNumber = None  # (String(30))
        self.faxNumber = None  # (String(30))
        self.taxCodeId = None  # (Integer)
        self.expenseReimbursement = None  # (Number)
        self.primaryAddressFlag = None  # (Boolean)
        self.defaultShippingFlag = None  # (Boolean)
        self.defaultBillingFlag = None  # (Boolean)
        self.defaultMailingFlag = None  # (Boolean)
        self.calendar = None  # (CalendarReference)
        self.timeZone = None  # (TimeZoneReference)
        self.company = None  # (CompanyReference)
        self._info = None  # (Metadata)
        
        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Order(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.company = None  # *(CompanyReference)
        self.contact = None  # (ContactReference)
        self.phone = None  # (String)
        self.phoneExt = None  # (String)
        self.email = None  # (String)
        self.site = None  # (SiteReference)
        self.status = None  # *(OrderStatusReference)
        self.opportunity = None  # (OpportunityReference)
        self.orderDate = None  # (String)
        self.dueDate = None  # (String)
        self.billingTerms = None  # (BillingTermsReference)
        self.taxCode = None  # (TaxCodeReference)
        self.poNumber = None  # (String(50))
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.salesRep = None  # *(MemberReference)
        self.notes = None  # (String)
        self.billClosedFlag = None  # (Boolean)
        self.billShippedFlag = None  # (Boolean)
        self.restrictDownpaymentFlag = None  # (Boolean)
        self.description = None  # (String)
        self.topCommentFlag = None  # (Boolean)
        self.bottomCommentFlag = None  # (Boolean)
        self.shipToCompany = None  # (CompanyReference)
        self.shipToContact = None  # (ContactReference)
        self.shipToSite = None  # (SiteReference)
        self.billToCompany = None  # (CompanyReference)
        self.billToContact = None  # (ContactReference)
        self.billToSite = None  # (SiteReference)
        self.productIds = None  # (Integer[])
        self.documentIds = None  # (Integer[])
        self.invoiceIds = None  # (Integer[])
        self.configIds = None  # (Integer[])
        self.total = None  # (Number)
        self.taxTotal = None  # (Number)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /schedule/statuses
from connectpyse.schedule import schedule_status


class ScheduleStatusesAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'schedule'
        self.module = 'statuses'
        self._class = schedule_status.ScheduleStatus
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_schedule_statuses(self):
        return super()._get()

    def create_schedule_status(self, a_status):
        return super()._create(a_status)

    def get_schedule_statuses_count(self):
        return super()._get_count()

    def get_schedule_status_by_id(self, status_id):
        return super()._get_by_id(status_id)

    def delete_schedule_status_by_id(self, status_id):
        super()._delete_by_id(status_id)

    def replace_schedule_status(self, status_id):
        pass

    def update_schedule_status(self, status_id, key, value):
        return super()._update(status_id, key, value)

    def merge_schedule_status(self, a_status, target_status_id):
        # return super()._merge(a_schedule_status, target_schedule_status_id)
        pass



from ..cw_model import CWModel


class AccountingUnpostedInvoice(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.billingLogId = None  # (Integer)
        self.locationId = None  # (Integer)
        self.departmentId = None  # (Integer)
        self.company = None  # (CompanyReference)
        self.accountNumber = None  # (String)
        self.billToCompany = None  # (CompanyReference)
        self.billToSite = None  # (SiteReference)
        self.shipToCompany = None  # (CompanyReference)
        self.shipToSite = None  # (SiteReference)
        self.invoiceNumber = None  # (String)
        self.invoiceDate = None  # (String)
        self.invoiceType = None  # (Enum)
        self.description = None  # (String)
        self.billingTerms = None  # (BillingTermsReference)
        self.dueDays = None  # (String)
        self.dueDate = None  # (String)
        self.currency = None  # (CurrencyReference)
        self.subTotal = None  # (Number)
        self.total = None  # (Number)
        self.invoiceTaxableFlag = None  # (Boolean)
        self.taxCode = None  # (TaxCodeReference)
        self.avalaraTaxFlag = None  # (Boolean)
        self.itemTaxableFlag = None  # (Boolean)
        self.salesTaxAmount = None  # (Number)
        self.stateTaxFlag = None  # (Boolean)
        self.stateTaxXref = None  # (String)
        self.stateTaxAmount = None  # (Number)
        self.countyTaxFlag = None  # (Boolean)
        self.countyTaxXref = None  # (String)
        self.countyTaxAmount = None  # (Number)
        self.cityTaxFlag = None  # (Boolean)
        self.cityTaxXref = None  # (String)
        self.cityTaxAmount = None  # (Number)
        self.countryTaxFlag = None  # (Boolean)
        self.countryTaxXref = None  # (String)
        self.countryTaxAmount = None  # (Number)
        self.compositeTaxFlag = None  # (Boolean)
        self.compositeTaxXref = None  # (String)
        self.compositeTaxAmount = None  # (Number)
        self.createdBy = None  # (String)
        self.dateClosed = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ProjectContact(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.projectId = None  # (Integer)
        self.contact = None  # *(ContactReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class KnowledgeBaseArticle(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.title = None  # *(String)
        self.issue = None  # *(String)
        self.resolution = None  # *(String)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.boardId = None  # (Integer)
        self.categoryId = None  # (Integer)
        self.subCategoryId = None  # (Integer)
        self.dateCreated = None  # (String)
        self.createdBy = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from connectpyse import restapi
# Class for /company/agreements
from connectpyse.finance import agreement


class AgreementsAPI(restapi.Client):

    module_url = 'finance'

    def __init__(self, base_url, **kwargs):
        super(AgreementsAPI, self).__init__('{}/{}'.format(base_url, AgreementsAPI.module_url), **kwargs)

    def get_agreements(self, user_headers, user_params=None):
        json_results = self.agreements.get(user_headers=user_headers, user_params=user_params)
        for json in json_results:
            yield agreement.Agreement(json)

    def create_agreement(self, a_agreement):
        pass

    def get_agreements_count(self, user_headers):
        json_results = self.agreements.get(the_id='count', user_headers=user_headers)
        count = json_results['count']
        return count

    def get_agreement_by_id(self, user_headers, agreement_id):
        a_agreement = agreement.Agreement(self.agreements.get(the_id=agreement_id, user_headers=user_headers))
        return a_agreement

    def delete_agreement_by_id(self, agreement_id   ):
        pass

    def replace_agreement(self, agreement_id):
        pass

    def update_agreement(self, agreement_id):
        pass

    def merge_agreement(self, agreement_id):
        pass

    def create_configuration_association(self, user_headers, agreement_id, config_id):
        # /agreements/{id}/configurations
        self.root_path = self.root_path + '/agreements/{}'.format(agreement_id)

        dict_post = {'id': config_id}
        json_results = self.configurations.post(user_data=dict_post, user_headers=user_headers)
        #      status_code = json_results.status_code
        return json_results

    def get_agreement_configurations(self):
        pass



from ..cw_controller import CWController
# Class for /company/contacts/{id}/communications
from connectpyse.company import contact_communication


class ContactCommunicationsAPI(CWController):
    def __init__(self, contact_id, **kwargs):
        self.module_url = 'company'
        self.module = 'contacts/{}/communications'.format(contact_id)
        self._class = contact_communication.ContactCommunication
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_contact_communications(self):
        return super()._get()

    def create_contact_communication(self, a_contact_communication):
        return super()._create(a_contact_communication)

    def get_contact_communications_count(self):
        return super()._get_count()

    def get_contact_communication_by_id(self, item_id):
        return super()._get_by_id(item_id)

    def delete_contact_communication_by_id(self, item_id):
        super()._delete_by_id(item_id)

    def replace_contact_communication(self, item_id):
        pass

    def update_contact_communication(self, item_id, key, value):
        return super()._update(item_id, key, value)



from ..cw_model import CWModel


class AccountingUnpostedProcurement(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.description = None  # (String)
        self.locationId = None  # (Integer)
        self.departmentId = None  # (Integer)
        self.procurementType = None  # (Enum)
        self.purchaseOrder = None  # (PurchaseOrderReference)
        self.purchaseDate = None  # (String)
        self.trackingNumber = None  # (String)
        self.billingTerms = None  # (BillingTermsReference)
        self.currency = None  # (CurrencyReference)
        self.total = None  # (Number)
        self.taxCode = None  # (TaxCodeReference)
        self.avalaraTaxFlag = None  # (Boolean)
        self.itemTaxableFlag = None  # (Boolean)
        self.purchaseOrderTaxableFlag = None  # (Boolean)
        self.stateTaxFlag = None  # (Boolean)
        self.stateTaxXref = None  # (String)
        self.stateTaxAmount = None  # (Number)
        self.countyTaxFlag = None  # (Boolean)
        self.countyTaxXref = None  # (String)
        self.countyTaxAmount = None  # (Number)
        self.cityTaxFlag = None  # (Boolean)
        self.cityTaxXref = None  # (String)
        self.cityTaxAmount = None  # (Number)
        self.countryTaxFlag = None  # (Boolean)
        self.countryTaxXref = None  # (String)
        self.countryTaxAmount = None  # (Number)
        self.compositeTaxFlag = None  # (Boolean)
        self.compositeTaxXref = None  # (String)
        self.compositeTaxAmount = None  # (Number)
        self.taxTotal = None  # (Number)
        self.customer = None  # (CompanyReference)
        self.vendor = None  # (CompanyReference)
        self.vendorAccountNumber = None  # (String)
        self.vendorInvoiceNumber = None  # (String)
        self.vendorInvoiceDate = None  # (String)
        self.taxFreightFlag = None  # (Boolean)
        self.freightTaxTotal = None  # (Number)
        self.freightCost = None  # (Number)
        self.dateClosed = None  # (String)
        self._info = None  # (Metadata)
        
        # initialize object with json dict
        super().__init__(json_dict)



import base64
import json
from os import path
import sys

from .config import *
from .cw_controller import *
#from .cw_info import all
from .cw_model import *
from .restapi import *

# sys.path.insert(0, path.dirname(path.abspath(__file__)))

# api_file = 'my_api.json'
# _api_file = path.join(path.dirname(path.abspath(__file__)), api_file)

# with open(_api_file) as fin:
#     cw_api_settings = json.load(fin)
# 
# API_URL = cw_api_settings['API_URL']
# _cid = cw_api_settings['COMPANYID']
# _pubk = cw_api_settings['PUBLICKEY']
# _privk = cw_api_settings['PRIVATEKEY']
# _authtoken = cw_api_settings['AUTHTOKEN']
# 
# if (_authtoken is not None) and (_authtoken != "xxxxx"):
#     basic_auth = _authtoken
# 
# else:
# 
#     basic_auth = base64.b64encode("{}+{}:{}".format(_cid, _pubk, _privk).encode('utf-8'))
#     basic_auth = {'Authorization': 'Basic {}'.format(str(basic_auth, 'utf-8'))}



from ..cw_model import CWModel


class CampaignEmailOpened(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.campaignId = None  # (Integer)
        self.contactId = None  # *(Integer)
        self.dateOpened = None  # (String)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ConfigurationTypeQuestion(CWModel):
    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.configurationTypeId = None  # (Integer)
        self.fieldType = None  # *(Enum)
        self.entryType = None  # *(Enum)
        self.sequenceNumber = None  # (Number)
        self.question = None  # *(String(1000))
        self.numberOfDecimals = None  # (Integer)
        self.requiredFlag = None  # (Boolean)
        self.inactiveFlag = None  # (Boolean)
        self.possibleAnswers = None  # (Array)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class OpportunityStatus(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(30))
        self.wonFlag = None  # (Boolean)
        self.lostFlag = None  # (Boolean)
        self.closedFlag = None  # (Boolean)
        self.inactiveFlag = None  # (Boolean)
        self.defaultFlag = None  # (Boolean)
        self._info = None  # (Metadata)
        self.enteredBy = None  # (String)
        self.dateEntered = None  # (String)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Currency(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.currencyIdentifier = None  # *(String(10))
        self.name = None  # (String(50))
        self.symbol = None  # (String(10))
        self.displayIdFlag = None  # (Boolean)
        self.displaySymbolFlag = None  # (Boolean)
        self.isoCode = None  # (String(3))
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class OpportunityTeam(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.type = None  # *(Enum)
        self.member = None  # (MemberReference)
        self.salesTeam = None  # (SalesTeamReference)
        self.commissionPercent = None  # (Integer)
        self.referralFlag = None  # (Boolean)
        self.opportunityId = None  # (Integer)
        self.responsibleFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /company/configurations
from . import configuration


class ConfigurationsAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'company'
        self.module = 'configurations'
        self._class = configuration.Configuration
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_configurations(self):
        return super()._get()

    def create_configuration(self, a_configuration):
        return super()._create(a_configuration)

    def get_configurations_count(self):
        return super()._get_count()

    def get_configuration_by_id(self, configuration_id):
        return super()._get_by_id(configuration_id)

    def delete_configuration_by_id(self, configuration_id):
        super()._delete_by_id(configuration_id)

    def replace_configuration(self, configuration_id):
        pass

    def update_configuration(self, configuration_id, key, value):
        return super()._update(configuration_id, key, value)




from ..cw_model import CWModel


class Survey(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.name = None  # *(String(50))
        self.inactiveFlag = None  # (Boolean)
        self.headerIncludeLogoFlag = None  # (Boolean)
        self.headerText = None  # (String(4000))
        self.headerTextVisibleFlag = None  # (Boolean)
        self.footerText = None  # (String(500))
        self.footerTextVisibleFlag = None  # (Boolean)
        self.thankYouText = None  # (String(4000))
        self.notifyWho = None  # (GenericIdIdentifierReference)
        self.notifyWhoVisibleFlag = None  # (Boolean)
        self.notifyMember = None  # (MemberReference)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class CompanyTeam(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.company = None  # (CompanyReference)
        self.teamRole = None  # *(TeamRoleReference)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.contact = None  # (ContactReference)
        self.member = None  # (MemberReference)
        self.accountManagerFlag = None  # (Boolean)
        self.techFlag = None  # (Boolean)
        self.salesFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /company/companies/{id}/managementSummaryReports
from ConnectPyse.company import company_management_summary_report


class CompanyManagementSummaryReportsAPI(CWController):
    def __init__(self, company_id, **kwargs):
        self.module_url = 'company'
        self.module = 'companies/{}/managementSummaryReports'.format(company_id)
        self._class = company_management_summary_report.CompanyManagementSummaryReport
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_managementSummaryReports(self):
        return super()._get()

    def create_managementSummaryReport(self, a_managementSummaryReport):
        return super()._create(a_managementSummaryReport)

    def get_managementSummaryReports_count(self):
        return super()._get_count()

    def get_managementSummaryReport_by_id(self, managementSummaryReport_id):
        return super()._get_by_id(managementSummaryReport_id)

    def delete_managementSummaryReport_by_id(self, managementSummaryReport_id):
        super()._delete_by_id(managementSummaryReport_id)

    def replace_managementSummaryReport(self, managementSummaryReport_id):
        pass

    def update_managementSummaryReport(self, managementSummaryReport_id, key, value):
        return super()._update(managementSummaryReport_id, key, value)



from ..cw_model import CWModel


class AccountingBatchTransaction(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.batch = None  # (BatchReference)
        self.billingLogId = None  # (Integer)
        self.invoiceNumber = None  # (String)
        self.type = None  # (Enum)
        self.description = None  # (String)
        self.cost = None  # (Number)
        self.item = None  # (String)
        self.inventory = None  # (String)
        self.salesCode = None  # (String)
        self.cogs = None  # (String)
        self.glAccount = None  # (String)
        self.currency = None  # (CurrencyReference)
        self.debitAmount = None  # (Number)
        self.creditAmount = None  # (Number)
        self.balance = None  # (String)
        self.product = None  # (ProductReference)
        self.quantity = None  # (Number)
        self.unitOfMeasure = None  # (UnitOfMeasureReference)
        self.serializedFlag = None  # (Boolean)
        self.serialNumber = None  # (String)
        self.shipmentMethod = None  # (ShipmentMethodReference)
        self.packingSlip = None  # (String)
        self.ticket = None  # (TicketReference)
        self.project = None  # (ProjectReference)
        self.phase = None  # (ProjectPhaseReference)
        self.expenseClass = None  # (Enum)
        self.expenseType = None  # (ExpenseTypeReference)
        self.timeEntry = None  # (TimeEntryReference)
        self.activity = None  # (ActivityReference)
        self.segment1 = None  # (String)
        self.segment2 = None  # (String)
        self.segment3 = None  # (String)
        self.segment4 = None  # (String)
        self.segment5 = None  # (String)
        self.segment6 = None  # (String)
        self.segment7 = None  # (String)
        self.segment8 = None  # (String)
        self.segment9 = None  # (String)
        self.segment10 = None  # (String)
        self.avalaraTaxFlag = None  # (Boolean)
        self.itemTaxableFlag = None  # (Boolean)
        self.taxCode = None  # (TaxCodeReference)
        self.stateTaxFlag = None  # (Boolean)
        self.stateTaxXref = None  # (String)
        self.stateTaxAmount = None  # (Number)
        self.countyTaxFlag = None  # (Boolean)
        self.countyTaxXref = None  # (String)
        self.countyTaxAmount = None  # (Number)
        self.cityTaxFlag = None  # (Boolean)
        self.cityTaxXref = None  # (String)
        self.cityTaxAmount = None  # (Number)
        self.countryTaxFlag = None  # (Boolean)
        self.countryTaxXref = None  # (String)
        self.countryTaxAmount = None  # (Number)
        self.compositeTaxFlag = None  # (Boolean)
        self.compositeTaxXref = None  # (String)
        self.compositeTaxAmount = None  # (Number)
        self.taxTotal = None  # (Number)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class Ticket(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.summary = None  # * (String(100))
        self.recordType = None  # (Enum)
        self.board = None  # (BoardReference)
        self.status = None  # (ServiceStatusReference)
        self.project = None  # (ProjectReference)
        self.phase = None  # (ProjectPhaseReference)
        self.wbsCode = None  # (String(50))
        self.company = None  # * (CompanyReference)
        self.site = None  # (SiteReference)
        self.siteName = None  # (String(50))
        self.addressLine1 = None  # (String(50))
        self.addressLine2 = None  # (String(50))
        self.city = None  # (String(50))
        self.stateIdentifier = None  # (String(50))
        self.zip = None  # (String(12))
        self.country = None  # (CountryReference)
        self.contact = None  # (ContactReference)
        self.contactPhoneNumber = None  # (String(20))
        self.contactPhoneExtension = None  # (String(15))
        self.contactEmailAddress = None  # (String(250))
        self.type = None  # (ServiceTypeReference)
        self.subType = None  # (ServiceSubTypeReference)
        self.item = None  # (ServiceItemReference)
        self.team = None  # (ServiceTeamReference)
        self.owner = None  # (MemberReference)
        self.priority = None  # (PriorityReference)
        self.serviceLocation = None  # (ServiceLocationReference)
        self.source = None  # (ServiceSourceReference)
        self.requiredDate = None  # (String)
        self.budgetHours = None  # (Number)
        self.opportunity = None  # (OpportunityReference)
        self.agreement = None  # (AgreementReference)
        self.severity = None  # (Enum)
        self.impact = None  # (Enum)
        self.externalXRef = None  # (String(100))
        self.poNumber = None  # (String(50))
        self.knowledgeBaseCategoryId = None  # (Integer)
        self.knowledgeBaseSubCategoryId = None  # (Integer)
        self.allowAllClientsPortalView = None  # (Boolean)
        self.customerUpdatedFlag = None  # (Boolean)
        self.automaticEmailContactFlag = None  # (Boolean)
        self.automaticEmailResourceFlag = None  # (Boolean)
        self.automaticEmailCcFlag = None  # (Boolean)
        self.automaticEmailCc = None  # (String(1000))
        self.initialDescription = None  # (String)
        self.initialInternalAnalysis = None  # (String)
        self.initialResolution = None  # (String)
        self.contactEmailLookup = None  # (String)
        self.processNotifications = None  # (Boolean)
        self.skipCallback = None  # (Boolean)
        self.closedDate = None  # (String)
        self.closedBy = None  # (String)
        self.closedFlag = None  # (Boolean)
        self.dateEntered = None  # (String)
        self.enteredBy = None  # (String)
        self.actualHours = None  # (Number)
        self.approved = None  # (Boolean)
        self.subBillingMethod = None  # (Enum)
        self.subBillingAmount = None  # (Number)
        self.subDateAccepted = None  # (String)
        self.dateResolved = None  # (String)
        self.dateResplan = None  # (String)
        self.dateResponded = None  # (String)
        self.resolveMinutes = None  # (Integer)
        self.resPlanMinutes = None  # (Integer)
        self.respondMinutes = None  # (Integer)
        self.isInSla = None  # (Boolean)
        self.knowledgeBaseLinkId = None  # (Integer)
        self.resources = None  # (String)
        self.parentTicketId = None  # (Integer)
        self.hasChildTicket = None  # (Boolean)
        self.knowledgeBaseLinkType = None  # (Enum)
        self.billTime = None  # (Enum)
        self.billExpenses = None  # (Enum)
        self.billProducts = None  # (Enum)
        self.predecessorType = None  # (Enum)
        self.predecessorId = None  # (Integer)
        self.predecessorClosedFlag = None  # (Boolean)
        self.lagDays = None  # (Integer)
        self.lagNonworkingDaysFlag = None  # (Boolean)
        self.estimatedStartDate = None  # (String)
        self.duration = None  # (Integer)
        self.locationId = None  # (Integer)
        self.businessUnitId = None  # (Integer)
        self.mobileGuid = None  # (Guid)
        self.sla = None  # (SLAReference)
        self._info = None  # (Metadata)
        self.customFields = None  # (Array)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class ContactType(CWModel):
    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.description = None  # *(String(50))
        self.defaultFlag = None  # (Boolean)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_model import CWModel


class TaxCode(CWModel):

    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.identifier = None  # *(String(8))
        self.description = None  # *(String(50))
        self.invoiceCaption = None  # *(String(25))
        self.country = None  # (CountryReference)
        self.effectiveDate = None  # *(String)
        self.defaultFlag = None  # (Boolean)
        self.displayOnInvoiceFlag = None  # (Boolean)
        self.canadaCalculateGSTFlag = None  # (Boolean)
        self.cancelDate = None  # (String)
        self.levelOneRate = None  # (Number)
        self.levelOneRateType = None  # (Enum)
        self.levelOneTaxableMax = None  # (Number)
        self.levelOneCaption = None  # (String(25))
        self.levelOneTaxCodeXref = None  # (String(50))
        self.levelOneAgencyXref = None  # (String(100))
        self.levelOneServicesFlag = None  # (Boolean)
        self.levelOneExpensesFlag = None  # (Boolean)
        self.levelOneProductsFlag = None  # (Boolean)
        self.levelOneApplySingleUnitFlag = None  # (Boolean)
        self.levelOneApplySingleUnitMin = None  # (Number)
        self.levelOneApplySingleUnitMax = None  # (Number)
        self.levelTwoRate = None  # (Number)
        self.levelTwoRateType = None  # (Enum)
        self.levelTwoTaxableMax = None  # (Number)
        self.levelTwoCaption = None  # (String(25))
        self.levelTwoTaxCodeXref = None  # (String(50))
        self.levelTwoAgencyXref = None  # (String(100))
        self.levelTwoServicesFlag = None  # (Boolean)
        self.levelTwoExpensesFlag = None  # (Boolean)
        self.levelTwoProductsFlag = None  # (Boolean)
        self.levelTwoApplySingleUnitFlag = None  # (Boolean)
        self.levelTwoApplySingleUnitMin = None  # (Number)
        self.levelTwoApplySingleUnitMax = None  # (Number)
        self.levelThreeRate = None  # (Number)
        self.levelThreeRateType = None  # (Enum)
        self.levelThreeTaxableMax = None  # (Number)
        self.levelThreeCaption = None  # (String(25))
        self.levelThreeTaxCodeXref = None  # (String(50))
        self.levelThreeAgencyXref = None  # (String(100))
        self.levelThreeServicesFlag = None  # (Boolean)
        self.levelThreeExpensesFlag = None  # (Boolean)
        self.levelThreeProductsFlag = None  # (Boolean)
        self.levelThreeApplySingleUnitFlag = None  # (Boolean)
        self.levelThreeApplySingleUnitMin = None  # (Number)
        self.levelThreeApplySingleUnitMax = None  # (Number)
        self.levelFourRate = None  # (Number)
        self.levelFourRateType = None  # (Enum)
        self.levelFourTaxableMax = None  # (Number)
        self.levelFourCaption = None  # (String(25))
        self.levelFourTaxCodeXref = None  # (String(50))
        self.levelFourAgencyXref = None  # (String(100))
        self.levelFourServicesFlag = None  # (Boolean)
        self.levelFourExpensesFlag = None  # (Boolean)
        self.levelFourProductsFlag = None  # (Boolean)
        self.levelFourApplySingleUnitFlag = None  # (Boolean)
        self.levelFourApplySingleUnitMin = None  # (Number)
        self.levelFourApplySingleUnitMax = None  # (Number)
        self.levelFiveRate = None  # (Number)
        self.levelFiveRateType = None  # (Enum)
        self.levelFiveTaxableMax = None  # (Number)
        self.levelFiveCaption = None  # (String(25))
        self.levelFiveTaxCodeXref = None  # (String(50))
        self.levelFiveAgencyXref = None  # (String(100))
        self.levelFiveServicesFlag = None  # (Boolean)
        self.levelFiveExpensesFlag = None  # (Boolean)
        self.levelFiveProductsFlag = None  # (Boolean)
        self.levelFiveApplySingleUnitFlag = None  # (Boolean)
        self.levelFiveApplySingleUnitMin = None  # (Number)
        self.levelFiveApplySingleUnitMax = None  # (Number)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from os import path
import json
import base64
import sys


sys.path.insert(0, path.dirname(path.abspath(__file__)))

api_file = 'my_api.json'
_api_file = path.join(path.dirname(path.abspath(__file__)), api_file)

try:
  with open(_api_file) as fin:
      cw_api_settings = json.load(fin)
  
  API_URL = cw_api_settings['API_URL']
  _cid = cw_api_settings['COMPANYID']
  _pubk = cw_api_settings['PUBLICKEY']
  _privk = cw_api_settings['PRIVATEKEY']
  _authtoken = cw_api_settings['AUTHTOKEN']
  _clientId = cw_api_settings['CLIENTID'] if 'CLIENTID' in cw_api_settings else None
  
  if (_authtoken is not None) and (_authtoken != "xxxxx"):
      basic_auth = _authtoken
  
  else:
      basic_auth = base64.b64encode("{}+{}:{}".format(_cid, _pubk, _privk).encode('utf-8'))
      basic_auth = {'Authorization': 'Basic {}'.format(str(basic_auth, 'utf-8'))}
  
  if (_clientId is not None):
    basic_auth['clientId'] = _clientId
except FileNotFoundError as e:
  API_URL = ''
  basic_auth = {}


from ..cw_controller import CWController
# Class for /sales/opportunity
from . import opportunity


class OpportunityAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'sales'
        self.module = 'opportunities'
        self._class = opportunity.Opportunity
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_opportunities(self):
        return super()._get()

    def create_opportunity(self, a_type):
        return super()._create(a_type)

    def get_opportunities_count(self):
        return super()._get_count()

    def get_opportunity_by_id(self, type_id):
        return super()._get_by_id(type_id)

    def delete_opportunity_by_id(self, type_id):
        super()._delete_by_id(type_id)

    def replace_opportunity(self, type_id):
        pass

    def update_opportunity(self, type_id, key, value):
        return super()._update(type_id, key, value)

    def merge_opportunity(self, a_type, target_type_id):
        # return super()._merge(a_type, target_type_id)
        pass



from ..cw_controller import CWController
# Class for /procurement/products
from . import product_item


class ProductItemAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'procurement'
        self.module = 'products'
        self._class = product_item.ProductItem
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_product_items(self):
        return super()._get()

    def create_product_item(self, a_productitem):
        return super()._create(a_productitem)

    def get_product_items_count(self):
        return super()._get_count()

    def get_product_item_by_id(self, product_item_id):
        return super()._get_by_id(product_item_id)

    def delete_product_item_by_id(self, product_item_id):
        super()._delete_by_id(product_item_id)

    def replace_product_item(self, product_item_id):
        pass

    def update_product_item(self, product_item_id, key, value):
        return super()._update(product_item_id, key, value)


from ..cw_model import CWModel


class ContactNote(CWModel):
    def __init__(self, json_dict=None):
        self.id = None  # (Integer)
        self.contactId = None  # (Integer)
        self.text = None  # *(String)
        self.type = None  # (NoteTypeReference)
        self.flagged = None  # (Boolean)
        self.enteredBy = None  # (String)
        self._info = None  # (Metadata)

        # initialize object with json dict
        super().__init__(json_dict)



from ..cw_controller import CWController
# Class for /schedule/reminderTimes
from connectpyse.schedule import schedule_reminder_time

''' not available until 2017.3 '''
class ScheduleReminderTimesAPI(CWController):
    def __init__(self, **kwargs):
        self.module_url = 'schedule'
        self.module = 'reminderTimes'
        self._class = schedule_reminder_time.ScheduleReminderTime
        super().__init__(**kwargs)  # instance gets passed to parent object

    def get_schedule_reminder_times(self):
        return super()._get()

    def create_schedule_reminder_time(self, a_reminder_time):
        return super()._create(a_reminder_time)

    def get_schedule_reminder_times_count(self):
        return super()._get_count()

    def get_schedule_reminder_time_by_id(self, reminder_time_id):
        return super()._get_by_id(reminder_time_id)

    def delete_schedule_reminder_time_by_id(self, reminder_time_id):
        super()._delete_by_id(reminder_time_id)

    def replace_schedule_reminder_time(self, reminder_time_id):
        pass

    def update_schedule_reminder_time(self, reminder_time_id, key, value):
        return super()._update(reminder_time_id, key, value)

    def merge_schedule_reminder_time(self, a_reminder_time, target_reminder_time_id):
        # return super()._merge(a_schedule_reminder_time, target_schedule_reminder_time_id)
        pass
